﻿using System;
using System.Windows;
using System.Windows.Media.Animation;
using System.Windows.Threading;

namespace Memento.Common.WaitControl
{
    /// <summary>
    /// 等待时钟控件
    /// </summary>
    public partial class WaitClock : Window
    {
        #region 成员变量
        
        private static WaitClock m_WaitClock;           // 等待时钟
        private static Window m_Parent;                 // 记录依附的父窗口
        private DateTime m_StartTime;                   // 起始时间
        private DispatcherTimer m_Timer;                // 计时器
        private Storyboard m_Storyboard;                // 文本动画

        #endregion
        
        #region 构造函数

        /// <summary>
        /// 等待时钟控件
        /// </summary>
        private WaitClock()
        {
            InitializeComponent();
        }
        
        #endregion

        #region 事件处理

        // 禁用右键菜单等操作
        private void WaitClock_MouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            e.Handled = true;
        }

        // 初始化并启动计时器
        private void WaitControl_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                m_StartTime = DateTime.Now;
                m_Timer = new DispatcherTimer();
                m_Timer.Tick += m_Timer_Tick;
                m_Timer.Start();

                m_Storyboard = (Storyboard)Resources["txtWaitStoryboard"];
                m_Storyboard.Begin(this);
            }
            catch (Exception ex)
            {
                MessageBox.Show("加载 WaitClock 控件异常，详细信息：\r\n" + ex.Message);
            }
        }

        // 更新时、分、秒针的角度
        private void m_Timer_Tick(object sender, EventArgs e)
        {
            string runTime = "";
            TimeSpan ts = DateTime.Now - m_StartTime;
            if (ts.Hours > 0)
            {
                runTime = ts.Hours + "时";
            }
            if (ts.Minutes > 0)
            {
                runTime += ts.Minutes + "分";
            }
            if (ts.Seconds > 0)
            {
                runTime += ts.Seconds + "秒";
            }
            txtRunTime.Text = runTime;

            secondRotate.Angle = DateTime.Now.Second * 6 + 90;
            minuteRotate.Angle = DateTime.Now.Minute * 6 + DateTime.Now.Second * 0.1 + 90;
            hourRotate.Angle = DateTime.Now.Hour * 30 + DateTime.Now.Minute * 0.5 + DateTime.Now.Second * (1.0 / 120) + 90;
        }

        #endregion

        #region 公开方法

        /// <summary>
        /// 显示等待时钟
        /// </summary>
        /// <param name="tipContent">提示信息</param>
        /// <param name="parent">依赖的父窗口，默认为 null 时表示依赖应用程序主窗口。</param>
        public static void ShowClock(string tipContent = "", Window parent = null)
        {
            if (parent == null)
            {
                parent = Application.Current.MainWindow;
            }
            m_Parent = parent;

            if (m_WaitClock == null)
            {
                m_WaitClock = new WaitClock();
            }
            m_WaitClock.txtTip.Text = tipContent;
            m_WaitClock.Owner = parent;                     // 设置依赖的父窗口
            m_WaitClock.Height = m_Parent.ActualHeight;     // 等待时钟窗口完全覆盖父窗口
            m_WaitClock.Width = m_Parent.ActualWidth;
            m_WaitClock.ShowDialog();
        }

        /// <summary>
        /// 关闭等待时钟
        /// </summary>
        public static void CloseClock()
        {
            if (m_WaitClock != null)
            {
                m_WaitClock.Dispatcher.BeginInvoke((Action)delegate ()
                {
                    m_WaitClock.m_Timer.Stop();
                    m_WaitClock.m_Storyboard.Stop();
                    m_WaitClock.Close();
                    m_WaitClock = null;

                    GC.Collect();
                    GC.WaitForFullGCComplete();
                });
            }
        }

        #endregion
    }
}